package communication;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import lejos.pc.comm.NXTConnector;

/**
 * Vytvari spojeni PC s NXT pres bluetooth.
 *
 * @author Michael
 */
public class Comm {
    
    private DataOutputStream dos;
    private DataInputStream dis;
    private NXTConnector conn;
    private boolean connected;

    /**
     * Konstruktor Bluetooth spojeni.
     */
    public Comm() {
        conn = new NXTConnector();
        dos = null;
        dis = null;
    }

    /**
     * Zda je navazane spojeni PC s NXT.
     * @return True pokud je spojeni.
     */
    public boolean isConnected() {
        return connected;
    }

    /**
     * Zavede spojeni s NXT pres bluetooth.
     * @return Zda se povedlo spojit.
     */
    public boolean connect() {
        connected = conn.connectTo("btspp://");
        dos = new DataOutputStream(conn.getOutputStream());
        dis = new DataInputStream(conn.getInputStream());
        return connected;
    }

    /**
     * * Vytvori a odesle packet se zpravou.
     * @param msg Zprava k odeslani.
     * @throws IOException
     */
    public void sendPacket(int msg) throws IOException {
        dos.writeInt(msg);
        dos.flush();
    }

    /**
     * Vytvori a odesle packet se zpravou.
     * @param msg Zprava k odeslani.
     * @throws IOException
     */
    public void sendPacket(String msg) throws IOException {
        dos.writeChars(msg);
        dos.flush();
    }

    /**
     * Ziska prichozi packet.
     * @return String zpravu z packetu.
     * @throws IOException
     */
    public String getPacket() throws IOException {
        return dis.readUTF();
    }

    /**
     * Uzavre odchozi stream.
     * @throws IOException
     */
    public void closeDos() throws IOException {
        dos.close();
    }

    /**
     * Uzavre prichozi stream.
     * @throws IOException
     */
    public void closeDis() throws IOException {
        dis.close();
    }

    /**
     * Uzavre BT spojeni s NXT.
     * @throws IOException
     */
    public void closeConnection() throws IOException {
        conn.close();
    }

    /**
     * Uzavre jak prichozi/odchozi stream, tak BT spojeni.
     * Odesle pred uzavrenim packet s hodnotou 1000, aby NXT radne uzavrel svuj
     * input stream.
     * @throws IOException
     */
    public void closeAll() throws IOException {
        sendPacket(1000);
        closeDis();
        closeDos();
        closeConnection();
    }
}
