package imageProcessing;

/**
 * Trida fungujici jako usecka v soustave souradnic. Popsana je pocatecnim bodem,
 * koncovym bodem a obecnou rovnici primky (a*x + b*y + c = 0).
 *
 * @author Michael
 */
public class Line {
    
    private static final float VZDALENOST_CAR = 8.0f; //vzdalenost car od sebe
    private static final float BOD_OD_CARY = 6.0f; //vzdalenost bodu od cary
    private static final float LIMIT_PRO_VYPOCET = 5.0f; //limitni vzdalenost pro aktualizaci abc
    
    private Point prvni;
    private Point posledni;
    
    private float a;
    private float b;
    private float c;
    
    /**
     * Konstruktor.
     * @param prvni Prvni bod popisujici usecku
     */
    public Line(Point prvni) {
        this.prvni = prvni;
        this.posledni = prvni;
        
        a = 0;
        b = 0;
        c = 0;
    }
    
    /**
     * Vrati prvni bod popisujici usecku.
     * @return Prvni bod
     */
    public Point getFirstPoint() {
        return prvni;
    }
    
    /**
     * Vrati posledni bod popisujici usecku.
     * @return Posledni bod
     */
    public Point getLastPoint() {
        return posledni;
    }
    
    /**
     * Vrati parametry a,b,c obecne rovnice primky (a*x + b*y + c = 0).
     * @return Vzdy pole 3 cisel. Prvni je a, druhe b a treti c.
     */
    public float[] getABC() {
        float[] arr = {a,b,c};
        return arr;
    }
    
    /**
     * Priradi bod primce. Pokud je bod na primce popsane obecnou rovnici primky,
     * tak jej priradi teto primce a posune tak posledni bod usecky pripadajici ji.
     * @param p Bod k pridani, pokud je vhodny
     * @return True pokud byl bod pridan
     */
    public boolean addPoint(Point p) {
        if(posledni.isNextToEachOther(p)) {
            if(prvni.distanceFrom(p) < LIMIT_PRO_VYPOCET) {
                //vypocet a,b,c
                posledni = p;
                countABC();
                return true;
            } else {
                //vymeni posledni bod
                if(isOnLine(p)) {
                    posledni = p;
                    return true;
                } else {
                    return false;
                }
            }
        } else {
            return false;
        }
    }
    
    /**
     * Otestuje, zda primka navazuje na tuto. Maximalni vzdalenost 2 primek je
     * limitovana parametrem "VZDALENOST_CAR".
     * @param l Cara pro porovnani
     * @return True pokud na sebe navazuji
     */
    public boolean isConnected(Line l) {
        float delkaUsecky = prvni.distanceFrom(posledni) + VZDALENOST_CAR;
        
        if(isOnLine(l.getFirstPoint())) {
            return ((prvni.distanceFrom(l.getFirstPoint()) < delkaUsecky) &&
                (posledni.distanceFrom(l.getFirstPoint()) < delkaUsecky));
        } else if(isOnLine(l.getLastPoint())) {
            return ((prvni.distanceFrom(l.getLastPoint()) < delkaUsecky) &&
                (posledni.distanceFrom(l.getLastPoint()) < delkaUsecky));
        } else {
            return false;
        }
    }
    
    /**
     * Zjisiti, zda jsou 2 cary stejne.
     * @param l Cara pro porovnani
     * @return True pokud se jedna o stejnou caru
     */
    public boolean isSame(Line l) {
        return prvni.isSame(l.getFirstPoint()) && posledni.isSame(l.getLastPoint());
    }
    
    /**
     * Vypocita koeficienty a,b,c obecne rovnice primky (a*x + b*y + c = 0).
     */
    public void countABC() {
        a = posledni.getY() - prvni.getY();
        b = -(posledni.getX() - prvni.getX());
        c = -(a * prvni.getX()) - (b * prvni.getY());
    }
    
    /**
     * Zda bod lezi primce popisujici caru, nebo je od ni vdalen maximalne
     * o "BOD_OD_CARY".
     * @param p Bod pro vyhodnoceni
     * @return True pokud je na primce
     */
    private boolean isOnLine(Point p) {
        float n = (float) ((Math.abs((a*p.getX())+(b*p.getY())+c))/Math.sqrt((a*a)+(b*b)));
        return (n < BOD_OD_CARY) && (n > -BOD_OD_CARY);
    }
    
}
