package imageProcessing;

/**
 * Trida fungujici jako bod v soustave souradnic.
 * K sestavovani car a nasledne tvaru ze zkoumaneho obrazku.
 *
 * @author Michael
 */
public class Point {
    
    private static final float OKOLI = 3.0f; //limit vzdalenosti 2 bodu od sebe
    
    private float x;
    private float y;
    
    /**
     * Konstruktor
     * @param x Pozice bodu v ose X
     * @param y Pozice bodu v ose Y
     */
    public Point(float x, float y) {
        this.x = x;
        this.y = y;
    }
    
    /**
     * Vrati souradnici na ose X.
     * @return Pozice na ose X
     */
    public float getX() {
        return x;
    }
    
    /**
     * Vrati souradnici na ose Y.
     * @return Pozice na ose Y
     */
    public float getY() {
        return y;
    }
    
    /**
     * Nastavi souradnice bodu.
     * @param x Pozice na ose X
     * @param y Pozice na ose Y
     */
    public void setPosition(float x, float y) {
        this.x = x;
        this.y = y;
    }

    /**
     * Zjisti, zda jsou body stejne.
     * @param bod Bod pro porovnani
     * @return True pokud jsou stejne
     */    
    public boolean isSame(Point bod) {
        return (x == bod.getX()) && (y == bod.getY());
    }
    
    /**
     * Zjisti, zda jsou 2 body vedle sebe. Vzdaleni od sebe o max "OKOLI" v jakemkoli
     * smeru.
     * @param p Bod pro porovnani
     * @return True pokud jsou vedle sebe
     */
    public boolean isNextToEachOther(Point p) {
        float x = p.getX() - this.x;
        float y = p.getY() - this.y;
        return !(((x < -OKOLI) || (x > OKOLI)) || ((y < -OKOLI) || (y > OKOLI)));
    }
    
    /**
     * Pocita vzdalenost 2 bodu od sebe.
     * @param p Bod pro porovnani
     * @return Vzdalenost
     */
    public float distanceFrom(Point p) {
        return (float) Math.sqrt(((p.getX()-x)*(p.getX()-x)) + ((p.getY()-y)*(p.getY()-y)));
    }
    
}
