package webcam;

import com.github.sarxos.webcam.Webcam;
import com.github.sarxos.webcam.WebcamPanel;
import com.github.sarxos.webcam.WebcamResolution;
import java.awt.Dimension;
import view.GraphicUI;

/**
 * Hlavni trida spravujici kameru a naslednou analyzu obrazu.
 *
 * @author Michael
 */
public class Cam {
    
    private Dimension ds = new Dimension(450, 360);
    private Dimension cs = WebcamResolution.VGA.getSize();
    private Webcam webcam = Webcam.getDefault();
    private WebcamPanel webCamPanel = new WebcamPanel(webcam, ds, false);
    
    private Thread tScreen; //vlakno pro Snapshot
    private Snapshot screen;
    private boolean connected; //jestli je kamera zavedena nebo vypnuta
    
    /**
     * Konstruktor.
     * @param gui Graficke rozhrani
     */
    public Cam(GraphicUI gui) {
        webcam.setViewSize(cs);
        webCamPanel.setFillArea(true);
        
        screen = new Snapshot(this, gui);
        tScreen = new Thread(screen);
        tScreen.setDaemon(true);
        connected = false;
    }
    
//GETs
    
    /**
     * Vrati default kameru.
     * @return 
     */
    public Webcam getWebcam() {
        return webcam;
    }
    
    /**
     * Vrati panel kamery pro grafiku a vykreslovani v kamery v GUI.
     * @return 
     */
    public WebcamPanel getWebcamPanel() {
        return webCamPanel;
    }
    
//public
    
    /**
     * Zda je kamera pripojena a aktivni.
     * @return True pokud je kamera aktivni
     */
    public boolean isConnected() {
        return connected;
    }

    /**
     * Pripoji kameru a spusti analyzu obrazu.
     */
    public void start() {
        if(!connected) {
            webCamPanel.start();
            tScreen.start();
            connected = true;
        }
    }
    
    /**
     * Zastavi kameru a analyzu obrazu.
     */
    public void stop() {
        if(connected) {
            try {
                tScreen.sleep(1);
                screen.requestStop();
                webCamPanel.stop();
            } catch (InterruptedException ex) {
                //nothing to do
            }
        }
    }
}
