package webcam;

import imageProcessing.ImageProcessor;
import imageProcessing.Line;
import imageProcessing.Shape;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import javax.imageio.ImageIO;
import view.GraphicUI;

/**
 * Trida delajici snapshoty z kamery a analyzujici hrany v nich. Vyuziva balicek
 * "imageProcessing".
 *
 * @author Michael
 */
public class Snapshot implements Runnable {
    
    private static final int DELAY = 2000; //frekvence analyzy obrazu
    
    private Cam cam;
    private GraphicUI gui;
    private ImageProcessor imgProc;
    private String name; //jmeno souboru pro nacitani obrazku
    
    private boolean canRun; //zda je mozno spusti vlakno, nebo jiz jede
    private int nShots; //kolikaty soubor
    private String textShapes;
    private static float divergence;
    
    /**
     * Konstruktor.
     * @param cam Pro tridu kamery, ze ktere cte obraz
     * @param gui Graficke rozhrani pro logovani
     */
    public Snapshot(Cam cam, GraphicUI gui) {
        this.cam = cam;
        this.gui = gui;
        imgProc = new ImageProcessor();
        name = "snapshot";
        
        canRun = true;
        nShots = 0;
        textShapes = "";
        divergence = 0.0f;
    }
    
// GETs/SETs
    
    /**
     * Ziska divergence, odchylku robota od idelani cesty.
     * @return 
     */
    public static float getDivergence() {
        return divergence;
    }
    
    /**
     * Nastavi nazev souboru pro ukladani a nacitani obrazku k analyze.
     * @param name 
     */
    public void setFileName(String name) {
        this.name = name;
        nShots = 0;
    }
    
// public
    
    /**
     * Pro ukonceni vlakna.
     */
    public void requestStop() {
        canRun = false;
    }
    
    /**
     * Zahajeni vlakna a spusteni analyzy.
     */
    @Override
    public void run() {
        while(canRun) {
            snapshot();
            gui.logShapes(textShapes);

            try {
                Thread.sleep(DELAY);
            } catch (InterruptedException ex) {
                //nothing to do
            }
        }
        
        canRun = true;
    }
    
//private
    
    /**
     * Cela logika pro nacteni originalniho obrazku, analyzu hran a naplneni
     * "textShapes".
     */
    private void snapshot() {
        String filePath = "img/" + name + nShots + ".jpg";
        
        //udela a analyzuje snapshot
        imgProc.setPathToImage(filePath);
        try {
            ImageIO.write(cam.getWebcam().getImage(), "JPG", new File(filePath));
            imgProc.findEdges();
        } catch (IOException ex) {
            System.out.println("File load err");
        }

        //analyza cesty
        imgProc.analyzePath();
        textShapes = imgProc.isLeftClear() ? "clear" : "obstacle";
        textShapes += " | ";
        textShapes += imgProc.isRightClear() ? "clear" : "obstacle";
        textShapes += "\n";
        divergence = imgProc.returnDivergence();
        
        //hledani tvaru
        ArrayList<Shape> al = imgProc.findShapes();
        textShapes += "Shapes: " + al.size() + "\n";
        for(int i = 0; i < al.size(); i++) {
            for(Line l : al.get(i).getLines()) {
                textShapes += 
                        i + "\n"
                        + l.getFirstPoint().getX() + " " + l.getFirstPoint().getY()
                        + "\n"
                        + l.getLastPoint().getX() + " " + l.getLastPoint().getY()
                        + "\n";
            }
        }
        
        //ulozeni 
        try {
            imgProc.writeEdgeFile();
        } catch(Exception e) {
            System.out.println("Write image err");
        }
        
        //nShots++; ///////////////////////////////NEZAPOMENOUT (odemknout pokud ma delat vice souboru)
    }
    
}
